/* SofChat (Web Chat program SHM version)      */
/* Copyright(c) 1996-1997 Y.YONEYAMA           */
/*                                             */
/* utils.c                                     */

#include <stdio.h>
#include "utils.h"

void sjis2euc(dst, src)
char *dst;
char *src;
{
  unsigned char c1, c2;
  char *d = dst;
  int i;
  for(i=0;i<strlen(src);i++)
  {
    c1 = src[i];
    if (!sjis1st(c1))
    {
      if(c1 >= 0xa0)
        *dst++ = 0x8e;
      *dst++ = c1;
    }
    else
    {
      c2 = src[i];
      c1 = src[++i];
      c2 = c2 + c2 - ((c2 <= 0x9f) ? 0xe1 : 0x161);
      if (c1 < 0x9f)
        c1 = c1 - ((c1 > 0x7f) ? ' ' : 0x1f);
      else {
        c1 = c1 - 0x7e;
        c2 ++;
      }
      *dst++ = c2 | 0x80;
      *dst++ = c1 | 0x80;
    }
  }
  *dst = '\0';
}

void euc2sjis(dst, src) /* including jis2sjis */
char *dst;
char *src;
{
  int c1, c2, i, k;
  char *d = dst;

  for (k=0,i=0; i<strlen(src);i++)
  {
    if((strlen(&src[i]) >= 3) && src[i] == ESC &&
       (src[i+1] == '$' || src[i+1] == '('))
    {
      if(src[i+1] == '$') k = 1;
      else k = 0;
      i += 3;
    }
    if((src[i] & 0x80) != 0 || k == 1)
    {
      if(src[i] == (char)0x8e)
      {
        i++;
        *dst++ = src[i];
      }
      else
      {
        unsigned char cc1, cc2;
        c2 = src[i] & 0x7f;
        c1 = src[++i] & 0x7f;
        cc2 = (((c2 - 1) >> 1) + ((c2 <= 0x5f) ? 0x71 : 0xb1));
        cc1 = (c1 + ((c2 & 1) ? ((c1 < 0x60) ? 0x1f : 0x20) : 0x7e));
        *dst++ = cc2;
        *dst++ = cc1;
      }
    }
    else
      *dst++ = src[i];
  }
  *dst = '\0';
}

void jis2euc(dst, src)
char *dst;
char *src;
{
  int kanji = 0;
  while(*src != '\0')
  {
    if(*src == ESC)
    {
      if(*(src+1) == '$')
      {
        kanji = 1;
        src+=3;
        continue;
      }
      else if(*(src+1) == '(')
      {
        kanji = 0;
        src+=3;
        continue;
      }
    }
    else
    {
      if(kanji)
      {
        *dst++ = *src++|0x80;
        *dst = *src|0x80;
      }
      else
        *dst = *src;
      src++;
      dst++;
    }
  }
  *dst = '\0';
}

int h2i(c)
char c;
{
  int d;
  if(c > '9')
    d = (c & 0x5f) - 'A' + 10;
  else
    d = c - '0';
  return(d);
}

char i2h(i)
int i;
{
  if(i < 10)
    return('0' + i);
  else
    return('A' + (i - 10));
}

int isIP(ip)
char *ip;
{
  int i, j = 0;
  for(i = 0; i < strlen(ip); i++)
  {
    if(ip[i] == '.')
    {
      j++;
      continue;
    }
    if(!(ip[i] >= '0' && ip[i] <= '9'))
      return 0;
  }
  if(j == 3) return 1;
  return 0;
}

/* -1: error */
IP str2IP(ip)
char *ip;
{
  char s[21];
  int i = 0, dots = 0;
  unsigned long addr = 0;

  while(1)
  {
    if(*ip == '.' || *ip == '\0')
    {
      s[i] = '\0';
      addr *= 256;
      addr += atoi(s);
      if(*ip == '\0') break;
      i = 0;
      dots ++;
      if(dots > 3) return(-1);
    }
    else
    {
      if((*ip >= '0') && (*ip <= '9'))
        s[i++] = *ip;
      else
        return(-1);
    }
    ip++;
  }
  if(dots != 3) return(-1);
  return(addr);
}

void i2ip(ip, i)
char *ip;
IP i;
{
  if(ip != NULL)
    sprintf(ip, "%d.%d.%d.%d", (i>>24)&255, (i>>16)&255, (i>>8)&255, i&255);
  else
    printf("%d.%d.%d.%d", (i>>24)&255, (i>>16)&255, (i>>8)&255, i&255);
}

int compareIP(ip1, ip2, mask)
IP ip1;
IP ip2;
IP mask;
{
  if((ip1&mask) == (ip2&mask))
    return(0);
  else
    return(1);
}

char *get_env(str)
char *str;
{
  static char *null = "";
  if(getenv(str) == NULL)
    return(null);
  return((char *)getenv(str));
}

char *jstrncpy(dst, src, max)
char *dst;
char *src;
int max;
{
  int i;
  for(i=0;i<(max-1) && *src != '\0';i++)
  {
    if(sjis1st(*src))
    {
      if(i >= max - 2) break;
      *dst++ = *src++;
      i++;
    }
    *dst++ = *src++;
  }
  *dst = '\0';
}

void scstrlwr(str)
char *str;
{
  int i;
  for(i = 0;i < strlen(str); i++)
    if(str[i] >= 'A' && str[i] <= 'Z') str[i] |= 0x20; /* = strlwr() */
}

